module RevelryData
  module Resource
    extend ActiveSupport::Concern

    included do
      include JSONAPI::ActsAsResourceController
      respond_to :html, :json, :props
      before_filter :setup_request

      def new
        if respond_to? :process_request_operations
          process_request_operations
        else
          process_request
        end
      end

      def setup_request
        @request = JSONAPI::Request.new(params, {
                                                context: context,
                                                key_formatter: key_formatter
                                              })
        render_errors(@request.errors) unless @request.errors.empty?
      rescue => e
        handle_exceptions(e)
      end

      def process_request
        @request = JSONAPI::Request.new(params, context: context,
                                        key_formatter: key_formatter,
                                        server_error_callbacks: (self.class.server_error_callbacks || []))
        unless @request.errors.empty?
          render_errors(@request.errors)
        else
          operation_results = create_operations_processor.process(@request)
          render_results(operation_results)
        end

      rescue => e
        handle_exceptions(e)
      ensure
        if response.body.size > 0 && fmt_is_json_api?
          response.headers['Content-Type'] = JSONAPI::MEDIA_TYPE
        end
      end

      def fmt_is_json_api?
        request.format == 'json'
      end

      def setup_response
        return unless fmt_is_json_api?

        if response.body.size > 0
          response.headers['Content-Type'] = JSONAPI::MEDIA_TYPE
        end
      end

      def render_results(operation_results)
        response_doc = create_response_document(operation_results)

        respond_with response_doc.contents do |fmt|
          fmt.html do
            @json_api = response_doc.contents
            render status: response_doc.status
          end
          fmt.json do
            render json: response_doc.contents, status: response_doc.status
          end
          fmt.props do
            render json: response_doc.contents, status: response_doc.status
          end
        end
      end

      def base_response_meta
        {
          current_user: current_user,
          csrf_token: form_authenticity_token,
          current_org: current_user.try(:org),
          flashes: flash,
        }
      end
    end
  end
end
